/*----------------------------------------------------------------------------
           PalmSens EmStat Pico Development Board
  ----------------------------------------------------------------------------
  Copyright (c) 2016, PalmSens BV

  All rights reserved.

  You have a royalty-free right to use, modify, reproduce and distribute
  the Sample Application Files (and/or any modified version) in any way
  you find useful, provided that you agree that PalmSens B.V. has no
  warranty, obligations or liability for any Sample Application Files.


  Demonstrates use of the 2-WIRE REAL-TIME CLOCK 'ABLIC S-35390A' using the Arduino MKRZERO
  Created 29 december 2018


  Wire Read/Write commands ar made up of the "Device Code" + "Command"
  Device Code of the RTC 'ABLIC S-35390A' : 0110 (6)
  The Device code is concatenate with the command code
  Example to request 3 byte from "Real-time data 2 access"
  Wire.requestFrom(address, quantity, stop)
  Wire.requestFrom(51, 3, 1);
  51 = The 7-bit address of the device to request bytes from ; 'Device Code 0110' + 'Command 011' = 0110011
  3  = The number of bytes to request
  1  = Stop : boolean. true will send a stop message after the request, releasing the bus.
             false will continually send a restart after the request, keeping the connection active.
  ----------------------------------------------------------------------------
*/

#include <Wire.h>

// Constant values which are filled in the function to set the ALARM "INT1 register access" Register
const int AlarmMinute = 30;     // Minute data (00 to 59)
const int AlarmHour = 9;       // Hour data (00 to 23)
const int AlarmWeek = 0;        //Day of the week data (00, Monday to 06,Sunday)
const char* DayOfTheWeek[] = {"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"};

// Constant values which are filled in the function to set the TIME "Real-time data 1 access" Register
const int TimeYear = 18;        // Year data (00 to 99)
const int TimeMonth = 12;       // Month data (01 to 12)
const int TimeDayOfMonth = 20;  // Day data (01 to 31)
const int TimeDayOfWeek = 3;    // Day of the week data (00 to 06)
const int TimeHour = 14;        // Hour data (00 to 23)
const int TimeMinute = 10;      // Minute data (00 to 59)
const int Timesecond = 0;      // Second data (00 to 59)
const char* Month[] = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};


/********************************************************************************************************************************************************/
// Reverse bit order of a Byte ( MSB "76543210" -> LSB "01234567" )
/********************************************************************************************************************************************************/
unsigned char ReverseByte(unsigned char b) {
  b = (b & 0xF0) >> 4 | (b & 0x0F) << 4;  // Swap 7654 3210 -> 3210 7654 | Mask 1111 0000 <-> 0000 1111
  b = (b & 0xCC) >> 2 | (b & 0x33) << 2;  // Swap 3210 7654 -> 1032 5476 | Mask 1100 1100 <-> 0011 0011
  b = (b & 0xAA) >> 1 | (b & 0x55) << 1;  // Swap 3210 7654 -> 0123 4567 | Mask 1010 1010 <-> 0101 0101
  return b;                               // Retrun Reversed Byte
}


/********************************************************************************************************************************************************/
// Print RTC "Real-time data 2 access" Register in a readable format Time : ( Hour:Minute:Second 00:00:00 )
/********************************************************************************************************************************************************/
void PrintRtcTime() {
  int i = 0;
  unsigned char cLO_NIBBLE[2];
  unsigned char cHI_NIBBLE[2];

  Wire.requestFrom(51, 3, 1);                                     // request 3 bytes from slave
  while (Wire.available()) {                                      // slave may send less than requested
    char cReadByte = Wire.read();                                 // receive a byte as character
    unsigned char cReversedReadByte = ReverseByte(cReadByte);     // Reverse bit order of the Byte
    cLO_NIBBLE[i] = (cReversedReadByte & 0x0F);                   // isolate the lower  4 bits of the byte for the single digits 00, 01, 02...09
    cHI_NIBBLE[i] = (cReversedReadByte & 0xF0) >> 4;              // isolate the higher 4 bits of the byte for the dubble digits 00, 10, 20...90
    i++;
  }
  if (cHI_NIBBLE[0] > 2) {
    cHI_NIBBLE[0] = (cHI_NIBBLE[0] & 0x3);                        // When the AM/PM bit is high, remove it from the byte for printing the time
  }
  Serial.print("Time : ");
  Serial.print(cHI_NIBBLE[0], DEC); // print the character
  Serial.print(cLO_NIBBLE[0], DEC); // print the character
  Serial.print(":");  // print the character
  Serial.print(cHI_NIBBLE[1], DEC); // print the character
  Serial.print(cLO_NIBBLE[1], DEC); // print the character
  Serial.print(":");  // print the character
  Serial.print(cHI_NIBBLE[2], DEC); // print the character
  Serial.print(cLO_NIBBLE[2], DEC); // print the character
  Serial.println(" ");
}


/********************************************************************************************************************************************************/
// Print RTC "Real-time data 1 access" Register in a readable format Time : ( DayOfTheWeek:Hour:Minut 00:00:00 )
/********************************************************************************************************************************************************/
void PrintRtcYear() {
  int i = 0;
  unsigned char cLO_NIBBLE[6];
  unsigned char cHI_NIBBLE[6];

  Wire.requestFrom(50, 7, 1);                                 // request 7 bytes from register 50
  while (Wire.available()) {                                  // slave may send less than requested
    char cReadByte = Wire.read();                             // receive a byte as character
    unsigned char cReversedReadByte = ReverseByte(cReadByte); // Reverse bit order of the Byte
    cLO_NIBBLE[i] = (cReversedReadByte & 0x0F);               // isolate the lower  4 bits of the byte for the single digits 00, 01, 02...09
    cHI_NIBBLE[i] = (cReversedReadByte & 0xF0) >> 4;          // isolate the higher 4 bits of the byte for the dubble digits 00, 10, 20...90
    i++;

  }
  if (cHI_NIBBLE[4] > 2) {                  // When the AM/PM bit is high, remove it from the byte for printing the time
    cHI_NIBBLE[4] = (cHI_NIBBLE[4] & 0x3);
  }
  Serial.print("Time of Day : ");
  Serial.print(cHI_NIBBLE[4], DEC);
  Serial.print(cLO_NIBBLE[4], DEC);
  Serial.print(":");
  Serial.print(cHI_NIBBLE[5], DEC);
  Serial.print(cLO_NIBBLE[5], DEC);
  Serial.print(":");
  Serial.print(cHI_NIBBLE[6], DEC);
  Serial.print(cLO_NIBBLE[6], DEC);

  Serial.print("\t");
  Serial.print(cHI_NIBBLE[2], DEC);
  Serial.print(cLO_NIBBLE[2], DEC);
  Serial.print("/");
  Serial.print(cHI_NIBBLE[1], DEC);
  Serial.print(cLO_NIBBLE[1], DEC);
  Serial.print("/");
  Serial.print("20");
  Serial.print(cHI_NIBBLE[0], DEC);
  Serial.print(cLO_NIBBLE[0], DEC);
  Serial.println(" ");
}


/********************************************************************************************************************************************************/
// Read intterupt status from "Status register 1 access"
/********************************************************************************************************************************************************/
int ReadInterruptStatus() {
  unsigned char StatusReg2[8];

  Wire.requestFrom(48, 1, 1);     // request 1 bytes from slave device #6 Command 000
  while (Wire.available()) {      // slave may send less than requested
    char cReadByte = Wire.read(); // receive a byte as character
    for (int i = 0; i < 8; i++) {
      StatusReg2[i] = (cReadByte & (i + 1) * 2) >> i;
    }
  }
  // Print Status as binairy string to the Serial Monitor.
  if (StatusReg2[3] == 1) { 
    Serial.print("\n\nAlarm time reached - Interrupt triggerd\n");
    Serial.print("Status register 1 : MSB | ");
    for (int i = 7; i >= 0; i--) {
      Serial.print(StatusReg2[i], BIN);
    }
    Serial.print(" | LSB\n");
    return 1;
  }
  else {
    return 0;
  }
}

/********************************************************************************************************************************************************
/ Enable the ALARM in "Status register 2 access" and fill in the specific value's in "INT1 register access".
/ In the example that is given the ALARM has been set to Generate a interrupt every day @ 09:30 (24 Hour clock) - Given that the TimeOfDay is set correct
/********************************************************************************************************************************************************/
void SetAlarm() {

  Wire.beginTransmission(49); // Start a write transaction to Register 49 - Status register 2 access
  Wire.write(0);              // Write 0 - Resetting the Intterupt enable bit.
  Wire.endTransmission();     // End write transaction

  Serial.print("\nReseting RTC & Alarm");
  int Int1 = ReadInterruptStatus();

  Wire.beginTransmission(49); // Start a write transaction to Register 49 - Status register 2 access
  Wire.write(32);             // Write 0010 0000 - setting INTAE "INTerupt Alarm Enable" to 1.
  Wire.endTransmission();     // End write transaction

  unsigned char ReverseAlarmWeek   = FormatTime(AlarmWeek);   // Formats the data to the RTC format. bit 7 |W1|W2|W4| -    -  | - |  -  | - | bit 0
  unsigned char ReverseAlarmHour   = FormatTime(AlarmHour);   // Formats the data to the RTC format. bit 7 |H1|H2|H4|H8   H10 |H20|AM/PM| - | bit 0
  unsigned char ReverseAlarmMinute = FormatTime(AlarmMinute); // Formats the data to the RTC format. bit 7 |m1|m2|m4|m8   m10 |m20| m40 | - | bit 0

  Serial.print("\nAlarm is set to : ");
  if (ReverseAlarmWeek > 0) {                     // When there is a value in the Week register
    ReverseAlarmWeek = ReverseAlarmWeek + 1;      // Enable the Week register by setting bit B0(A2WE) high.
  }
  if (ReverseAlarmHour > 0) {                     // When there is a value in the Hour register
    ReverseAlarmHour = ReverseAlarmHour + 1;      // Enable the Hour register by setting bit B0(A2HE) high.
  }
  if (ReverseAlarmMinute > 0) {                   // When there is a value in the Minute register
    ReverseAlarmMinute = ReverseAlarmMinute + 1;  // Enable the Minute register by setting bit B0(A2mE) high.
  }

  // Print information about the time that have been set in the ALARM
  Serial.print(DayOfTheWeek[AlarmWeek]);
  Serial.print(", " );
  Serial.print(AlarmHour, DEC);
  Serial.print(":" );
  Serial.print(AlarmMinute, DEC);

  //  In 24-hour mode, users can write either 0 or 1. 0 is read when the hour data is from 00 to 11, and 1 is read when from 12 to 23.
  if (AlarmHour > 11) {
    ReverseAlarmHour = ReverseAlarmHour + 2; // Enabling the AM/PM bit of the register
  }

  Wire.beginTransmission(52);     // starting a write sequence to register 52 (INT1 register access). Important to set INT1AE = 1, INT1ME = 0,INT1FE = 0 in 'Status register 2 access' to be able to set a alarm time.
  Wire.write(ReverseAlarmWeek);   // Set Week & enable the Week register by setting bit B0(A2WE) high.      (Example has been set to 0 weeks)
  Wire.write(ReverseAlarmHour);   // Set Hour & enable the Hour register by setting bit B0(A2HE) high.      (Example has been set to 0 hours)
  Wire.write(ReverseAlarmMinute); // Set Minute & enable the minute register by setting bit B0(A2mE) high.  (Example has been set to 1 minute)
  Wire.endTransmission();         // stop transmitting
}

/********************************************************************************************************************************************************/
// Format Time to "RTC format" - > bit 7 |1|2|4|8|10|20|40|80| bit 0
/********************************************************************************************************************************************************/
unsigned char FormatTime(unsigned char x) {
  unsigned char cLO_NIBBLE = 0;
  unsigned char cHI_NIBBLE = 0;
  while (x != 0) {
    if (x < 10) {
      cLO_NIBBLE = x;
      x = (x - x);
      cLO_NIBBLE = (cLO_NIBBLE & 0xC) >> 2 | (cLO_NIBBLE & 0x3) << 2;
      cLO_NIBBLE = (cLO_NIBBLE & 0xA) >> 1 | (cLO_NIBBLE & 0x5) << 1;
    } else if (x >= 10 && x < 20) {
      cHI_NIBBLE = 8;
      x = x - 10;
    } else if (x >= 20 && x < 40) {
      cHI_NIBBLE = cHI_NIBBLE + 4;
      x = x - 20;
    } else if (x >= 40 && x < 80) {
      cHI_NIBBLE = cHI_NIBBLE + 2;
      x = x - 40;
    } else if (x >= 80) {
      cHI_NIBBLE = cHI_NIBBLE + 1;
      x = x - 80;
    }
  }
  unsigned char FormatedTime = (cLO_NIBBLE & 0x0F) << 4 | (cHI_NIBBLE & 0x0F);
  return FormatedTime;
}

/********************************************************************************************************************************************************/
// Sets the Time Of Day. The constant int values are used to set the Time Of Day.
/********************************************************************************************************************************************************/
void SetTimeOfDay() {

  unsigned char ReverseTimeYear       = FormatTime(TimeYear);       // Formats the data to the RTC format. bit 7 |Y1|Y2|Y4|Y8   Y10 |Y20| Y40 |Y80| bit 0
  unsigned char ReverseTimeMonth      = FormatTime(TimeMonth);      // Formats the data to the RTC format. bit 7 |M1|M2|M4|M8   M10 |-  |  -  | - | bit 0
  unsigned char ReverseTimeDayOfMonth = FormatTime(TimeDayOfMonth); // Formats the data to the RTC format. bit 7 |D1|D2|D4|D8   D10 |D20|  -  | - | bit 0
  unsigned char ReverseTimeDayOfWeek  = FormatTime(TimeDayOfWeek);  // Formats the data to the RTC format. bit 7 |W1|W2|W4| -    -  | - |  -  | - | bit 0
  unsigned char ReverseTimeHour       = FormatTime(TimeHour);       // Formats the data to the RTC format. bit 7 |H1|H2|H4|H8   H10 |H20|AM/PM| - | bit 0
  unsigned char ReverseTimeMinute     = FormatTime(TimeMinute);     // Formats the data to the RTC format. bit 7 |m1|m2|m4|m8   m10 |m20| m40 | - | bit 0
  unsigned char ReverseTimesecond     = FormatTime(Timesecond);     // Formats the data to the RTC format. bit 7 |s1|s2|s4|s8   s10 |s20| s40 | - | bit 0

  if (TimeHour > 11) {
    ReverseTimeHour = (ReverseTimeHour + 2);
  }

  Wire.beginTransmission(50);         // starting a write sequence to register 52 (INT1 register access). Important to set INT1AE = 1, INT1ME = 0,INT1FE = 0 in 'Status register 2 access' to be able to set a alarm time.
  Wire.write(ReverseTimeYear);        // Set Year
  Wire.write(ReverseTimeMonth);       // set Month
  Wire.write(ReverseTimeDayOfMonth);  // set day of month
  Wire.write(ReverseTimeDayOfWeek);   // set day of week
  Wire.write(ReverseTimeHour);        // set hour
  Wire.write(ReverseTimeMinute);      // set minute
  Wire.write(ReverseTimesecond);      // set second
  Wire.endTransmission();             // stop transmitting
}

/********************************************************************************************************************************************************/
// Setup code. Place any code here that only need to be executed one time.
/********************************************************************************************************************************************************/
void setup() {
  // Execute only one time
  Wire.begin();                 // join i2c bus (address optional for master)
  Serial.begin(9600);           // start serial for output

  Wire.beginTransmission(48);   // Write one Byte to Register 48 - "Status register 1 access"
  Wire.write(192);              // Reset RTC AND set to 24-hour "Write(1100 0000)"
  Wire.endTransmission();       // stop transmitting

  SetTimeOfDay();               // Will set the TimeOfDay to the value's that have been set in the "const int"
  SetAlarm();                   // Will set the ALARM to the value's that have been set in the "const int"
}

/********************************************************************************************************************************************************/
// Main Loop. Place code here that needs to be executed until infinit.
/********************************************************************************************************************************************************/
void loop() {
  int Int1 = ReadInterruptStatus(); // Reads the INT1 status.
  if (Int1 == 1) {
    SetAlarm();                     // When the interrup is triggered. Reset the alarm and set it again.
  }
  delay(1000);
  // PrintRtcTime();                 // Uncomment to print the Time to the Serial Monitor
  PrintRtcYear();                 // Uncomment to print the Time to the Serial Monitor

}
